@@ -1,13 +1,14 @@
-use crate::snapshot::Snapshot;
 use std::collections::HashMap;
 use std::sync::Mutex;
 
-use crate::data::cirru::code_to_calcit;
-use crate::primes::Calcit;
-use crate::snapshot;
-
 use cirru_parser::Cirru;
 
+use crate::data::cirru::code_to_calcit;
+use crate::primes::{Calcit, CalcitItems};
+use crate::snapshot;
+use crate::snapshot::Snapshot;
+use crate::util::string::extract_pkg_from_def;
+
 pub type ProgramEvaledData = HashMap<String, HashMap<String, Calcit>>;
 
 /// defRule: ns def
@@ -28,6 +29,9 @@ pub type ProgramCodeData = HashMap<String, ProgramFileData>;
 
 lazy_static! {
   static ref PROGRAM_EVALED_DATA_STATE: Mutex<ProgramEvaledData> = Mutex::new(HashMap::new());
+  // TODO need better soution for immediate calls
+  /// to be read by external logics and used as FFI
+  static ref PROGRAM_FFI_MESSAGES: Mutex<Vec<(String, CalcitItems)>> = Mutex::new(vec![]);
 }
 
 fn extract_import_rule(nodes: &Cirru) -> Result<Vec<(String, ImportRule)>, String> {
@@ -216,10 +220,45 @@ pub fn apply_code_changes(base: &ProgramCodeData, changes: &snapshot::ChangesDic
   Ok(program_code)
 }
 
-/// clear all states for now, very little defs are surely reused
-pub fn clear_all_program_evaled_defs() -> Result<(), String> {
-  // TODO might cache for performance, but also prints warning when ns and macros change
+/// clear evaled data after reloading
+pub fn clear_all_program_evaled_defs(init_fn: &str, reload_fn: &str, reload_libs: bool) -> Result<(), String> {
   let program = &mut PROGRAM_EVALED_DATA_STATE.lock().unwrap();
-  program.clear();
+  if reload_libs {
+    program.clear();
+  } else {
+    // reduce changes of libs. could be dirty in some cases
+    let init_pkg = extract_pkg_from_def(init_fn).unwrap();
+    let reload_pkg = extract_pkg_from_def(reload_fn).unwrap();
+    let mut to_remove: Vec<String> = vec![];
+    for k in program.keys() {
+      if k == &init_pkg
+        || k == &reload_pkg
+        || k.starts_with(&format!("{}.", init_pkg))
+        || k.starts_with(&format!("{}.", reload_pkg))
+      {
+        to_remove.push(k.to_owned());
+      } else {
+        continue;
+      }
+    }
+    for k in to_remove {
+      program.remove(&k);
+    }
+  }
   Ok(())
 }
+
+pub fn send_ffi_message(op: String, items: CalcitItems) {
+  let ref_messages = &mut PROGRAM_FFI_MESSAGES.lock().unwrap();
+  ref_messages.push((op, items))
+}
+
+pub fn take_ffi_messages() -> Result<Vec<(String, CalcitItems)>, String> {
+  let mut messages: Vec<(String, CalcitItems)> = vec![];
+  let ref_messages = &mut PROGRAM_FFI_MESSAGES.lock().unwrap();
+  for m in ref_messages.iter() {
+    messages.push(m.to_owned())
+  }
+  ref_messages.clear();
+  Ok(messages)
+}
